" ---------------------
" BBCODE SYNTAX FOR VIM
" ---------------------
" Original version from Georg Dahn (2008)
" Modified by PowerGod (2016)
"
" * Ver. 1.0.2 BETA
"
" - Changed the [code] recognition to give less issues (because of this, the tags now have a different color than the others)
" - Improved matchit.vim support, now it recognises better the correspondig tags (even with spaces), it is case insensitive, and also ignores tags in a [code] block 
" - Improved font style highlighting considering that [url] and [code] can be inside them
" - Corrected font style behaviour while inside [list]
" - All tag highlight should now ignore spaces 
"
" * Ver. 1.0.1 BETA
"
" - Added matchit.vim support
" - Removed highlight of tags inside [code] blocks and fixed relative tag colors
" - Added ignorecase for tags (instead of using the global setting)
" - Added sync from start (could be slower for huge files, but without it many scripts can be wrongly highlighted) 
" - Fixed highlight of other tags inside [LIST] (the [*] tag is higlighted in a different way for now)
" - Many tags now ignore spaces 
"
" TODO:
" - Highlight closed tags without an opening/closing match
"
if version < 600
    syntax clear
elseif exists("b:current_syntax")
    finish
endif

" Global case insensitivity, but it's preferably to set it for every tag
"syn case ignore
syn spell toplevel

syn match bbcodeItem contained "\[\s*[-a-zA-Z0-9]\+"hs=s+1 contains=@NoSpell
syn match bbcodeItem contained "\[\s*/\s*[-a-zA-Z0-9]\+"hs=s+2 contains=@NoSpell
" Removed because it was messing with list items
"syn match bbcodeItem contained "\[\s*\*\s*\]"hs=s+1,he=e-1 contains=@NoSpell
syn match bbcodeArgument contained "\s*[-a-zA-Z0-9]\+\s*="ms=s+1,me=e-1 contains=@NoSpell
syn region bbcodeValue contained start="\"" end="\"" contains=@NoSpell
syn region bbcodeValue contained start="'" end="'" contains=@NoSpell
syn match bbcodeValue contained "=\s*[\t ]*[^'" \t\]][^ \t\]]*"hs=s+1 contains=@NoSpell
syn region bbcodeTag start="\[\s*/\{0,1}\s*" end="\]" contains=@NoSpell,bbcodeItem,bbcodeArgument,bbcodeValue
"syn match bbcodeTag "\[\s*/\{0,1}.\{-}\s*\]" contains=@NoSpell,bbcodeItem,bbcodeArgument,bbcodeValue
" Required for correct highlight of [code] tag
"syn region bbcodeCodeTag start="\[\s*/\{0,1}\s*\ccode" end="\]" contains=@NoSpell,bbcodeItem,bbcodeArgument,bbcodeValue
"syn region bbcodeCodeIgnore transparent contained start="\]" end="\[\s*/\s*code\s*\]" contains=@NoSpell

syn region bbcodeUrl start="\[\s*\curl\s*[=\]]" end="\[\s*/\s*\curl\s*\]"me=e-6 contains=@NoSpell,bbcodeTag

" syn region bbcodeCode matchgroup=Delimiter start="/\[code\(=.\{-}\)\?\]/" end="/\[\/code\]/"
"syn region bbcodeCode matchgroup=Delimiter start="\[code\(=.\{-}\)\?\]"he=e+6 end="\[\/code\]"me=e-7
"syn region bbcodeCode matchgroup=Statement start="\[\ccode\(=.\{-}\)\?\]"he=e+6 end="\[/\ccode\]"me=e-7 contains=@NoSpell,bbcodeCodeTag
"syn region bbcodeCode matchgroup=Statement start="\[\ccode\(=.\{-}\)\?\]" end="\[/\ccode\]"me=e-7 contains=@NoSpell,bbcodeCodeTag
"syn region bbcodeCode start="\[\s*\ccode\s*\(=\s*.\{-}\)\?\]" end="\[\s*/\s*\ccode\s*\]"me=e-7 contains=@NoSpell,bbcodeCodeTag
"syn region bbcodeCode start="\[\s*\ccode\s*\(=\s*.\{-}\)\?\]"hs=e+1 end="\[\s*/\s*\ccode\s*\]"he=s-1 contains=@NoSpell,bbcodeCodeTag keepend
syn region bbcodeCode matchgroup=bbcodeTag start="\[\s*\ccode\s*\(=\s*.\{-}\)\?\]" end="\[\s*/\s*\ccode\s*\]" contains=@NoSpell keepend
"syn match bbcodeCode "\[\s*\ccode\s*\(=\s*.\{-}\)\?\].\{-}\[\s*/\s*\ccode\s*\]" contains=@NoSpell,bbcodeCodeTag

"syn region bbcodeBold start="\[\s*\cb\s*\]" end="\[\s*/\s*\cb\s*\]"me=e-4 contains=bbcodeTag,bbcodeBoldItalic,bbcodeBoldUnderline,bbcodeCode
syn region bbcodeBold start="\[\s*\cb\s*\]" end="\[\s*/\s*\cb\s*\]" contains=bbcodeTag,bbcodeBoldItalic,bbcodeBoldUnderline,bbcodeCode,bbcodeUrl keepend
"syn region bbcodeBold  start="\[\s*\cb\s*\]" end="\[/\cb\]" contains=bbcodeTag keepend
"syn region bbcodeBoldItalic contained start="\[\s*\ci\s*\]" end="\[\s*/\s*\ci\s*\]"me=e-4 contains=bbcodeTag,bbcodeBoldItalicUnderline,bbcodeCode
syn region bbcodeBoldItalic contained start="\[\s*\ci\s*\]" end="\[\s*/\s*\ci\s*\]" contains=bbcodeTag,bbcodeBoldItalicUnderline,bbcodeCode,bbcodeUrl keepend
"syn region bbcodeBoldItalicUnderline contained start="\[\s*\cu\s*\]" end="\[\s*/\s*\cu\s*\]"me=e-4 contains=bbcodeTag,bbcodeCode
syn region bbcodeBoldItalicUnderline contained start="\[\s*\cu\s*\]" end="\[\s*/\s*\cu\s*\]" contains=bbcodeTag,bbcodeCode,bbcodeUrl keepend
"syn region bbcodeBoldUnderline contained start="\[\s*\cu\s*\]" end="\[\s*/\s*\cu\s*\]"me=e-4 contains=bbcodeTag,bbcodeBoldUnderlineItalic,bbcodeCode
syn region bbcodeBoldUnderline contained start="\[\s*\cu\s*\]" end="\[\s*/\s*\cu\s*\]" contains=bbcodeTag,bbcodeBoldUnderlineItalic,bbcodeCode,bbcodeUrl keepend
"syn region bbcodeBoldUnderlineItalic contained start="\[\s*\ci\s*\]" end="\[\s*/\s*\ci\s*\]"me=e-4 contains=bbcodeTag,bbcodeCode
syn region bbcodeBoldUnderlineItalic contained start="\[\s*\ci\s*\]" end="\[\s*/\s*\ci\s*\]" contains=bbcodeTag,bbcodeCode,bbcodeUrl keepend

syn region bbcodeItalic start="\[\s*\ci\s*\]" end="\[\s*/\s*\ci\s*\]" contains=bbcodeTag,bbcodeItalicBold,bbcodeItalicUnderline,bbcodeCode,bbcodeUrl keepend
syn region bbcodeItalicBold contained start="\[\s*\cb\s*\]" end="\[\s*/\s*\cb\s*\]" contains=bbcodeTag,bbcodeItalicBoldUnderline,bbcodeCode,bbcodeUrl keepend
syn region bbcodeItalicBoldUnderline contained start="\[\s*\cu\s*\]" end="\[\s*/\s*\cu\s*\]" contains=bbcodeTag,bbcodeCode,bbcodeUrl keepend
syn region bbcodeItalicUnderline contained start="\[\s*\cu\s*\]" end="\[\s*/\s*\cu\s*\]" contains=bbcodeTag,bbcodeItalicUnderlineBold,bbcodeCode,bbcodeUrl keepend
syn region bbcodeItalicUnderlineBold contained start="\[\s*\cb\s*\]" end="\[\s*/\s*\cb\s*\]" contains=bbcodeTag,bbcodeCode,bbcodeUrl keepend

syn region bbcodeUnderline start="\[\s*\cu\s*\]" end="\[\s*/\s*\cu\s*\]" contains=bbcodeTag,bbcodeUnderlineBold,bbcodeUnderlineItalic,bbcodeCode,bbcodeUrl keepend
syn region bbcodeUnderlineBold contained start="\[\s*\cb\s*\]" end="\[\s*/\s*\cb\s*\]" contains=bbcodeTag,bbcodeUnderlineBoldItalic,bbcodeCode,bbcodeUrl keepend
syn region bbcodeUnderlineBoldItalic contained start="\[\s*\ci\s*\]" end="\[\s*/\s*\ci\s*\]" contains=bbcodeTag,bbcodeCode,bbcodeUrl keepend
syn region bbcodeUnderlineItalic contained start="\[\s*\ci\s*\]" end="\[\s*/\s*\ci\s*\]" contains=bbcodeTag,bbcodeUnderlineItalicBold,bbcodeCode,bbcodeUrl keepend
syn region bbcodeUnderlineItalicBold contained start="\[\s*\cb\s*\]" end="\[\s*/\s*\cb\s*\]" contains=bbcodeTag,bbcodeCode,bbcodeUrl keepend

" Definitions for matchit.vim (to jump from starting to closing tags) 
" (using % jumps from [LIST], [*] and [/LIST], else from the starting tag to the respective closing tag)
" (added match for "[]" in the end because else it wasn't working while jumping from "]")
let b:match_ignorecase = 1
let b:match_words = '\[\s*\cLIST\s*\(=.\{-}\)\?\]:\[\s*\*\s*\]:\[\s*/\s*\cLIST\s*\],\[\([^/]\{-}\)\(=.\{-}\)\?\]:\[\s*/\s*\1\s*\],\[:\]'

" CODE TO DO SOME TRIALS TO CATCH EXCEDING FINAL TAGS
"catch errors caused by wrong parenthesis
"syn region vfpParen transparent start='(' end=')' contains=ALLBUT,vfpParenErr
"syn match vfpParenErr ")"
"syn region vfpParen transparent start='\[[^/]\{-}\]' end='\[/.\{-}\]' contains=ALLBUT,vfpParenErr
"syn region vfpParen transparent start='\[\([^/]\{-}\)\]' end='\[\/\1\]' contains=ALLBUT,vfpParenErr
"syn match vfpParenErr "\[/.\{-}\]"
"syn region vfpParen transparent start='\[\([^/]\{-}\)\]' end='\[/\1\]' contains=ALLBUT,vfpParenErr
"syn match vfpParenErr "[/.*]"
"syn region vfpParen transparent start='\[' end='\]' contains=ALLBUT,vfpParenErr
"syn match vfpParenErr "\]"
"syn match vfpParenErr "\[\s*/.\{-}\]"
"syn region vfpParen transparent start='\[\([^/]\{-}\)\s*\(=.\{-}\)\?\]' end='\[\s*/\s*\1\s*\]' contains=vfpParen
"syn match vfpParenErr "\]"
"syn region vfpParen transparent start='\[' end='\]' contains=ALLBUT,vfpParenErr

" Define the default highlighting.
" For version 5.7 and earlier: only when not done already
" For version 5.8 and later: only when an item doesn't have highlighting yet
if version >= 508 || !exists("did_vfp_syn_inits")
	if version < 508
		let did_vfp_syn_inits = 1
		command -nargs=+ HiLink hi link <args>
	else
		command -nargs=+ HiLink hi def link <args>
	endif

	HiLink bbcodeTag Identifier
"	HiLink bbcodeCodeTag Identifier
	HiLink bbcodeItem Statement
	HiLink bbcodeArgument Type
	HiLink bbcodeValue Constant
	HiLink bbcodeUrl Underlined
	HiLink bbcodeCode PreProc
	HiLink vfpParenErr Error

	HiLink bbcodeBoldUnderlineItalic bbcodeBoldItalicUnderline
	HiLink bbcodeItalicBold bbcodeBoldItalic
	HiLink bbcodeItalicBoldUnderline bbcodeBoldItalicUnderline
	HiLink bbcodeItalicUnderlineBold bbcodeBoldItalicUnderline
	HiLink bbcodeUnderlineBold bbcodeBoldUnderline
	HiLink bbcodeUnderlineBoldItalic bbcodeBoldItalicUnderline
	HiLink bbcodeUnderlineItalic bbcodeItalicUnderline
	HiLink bbcodeUnderlineItalicBold bbcodeBoldItalicUnderline

	delcommand HiLink
endif

hi def bbcodeBold term=bold cterm=bold gui=bold
hi def bbcodeBoldItalic term=bold,italic cterm=bold,italic gui=bold,italic
hi def bbcodeBoldItalicUnderline term=bold,italic,underline cterm=bold,italic,underline gui=bold,italic,underline
hi def bbcodeBoldUnderline term=bold,underline cterm=bold,underline gui=bold,underline
hi def bbcodeItalic term=italic cterm=italic gui=italic
hi def bbcodeItalicUnderline term=italic,underline cterm=italic,underline gui=italic,underline
hi def bbcodeUnderline term=underline cterm=underline gui=underline

" This will make more precise the syntax check
" but could be commented out if it becomes too slow
syntax sync fromstart

let b:current_syntax = "bbcode"
